import humanize
import os
import os.path
import stat
from srht.config import cfg
from srht.database import db, DbSession
from srht.flask import SrhtFlask, session
from srht.oauth import AbstractOAuthService
from hgsrht.hgwebshim import HgWebShim
from hgsrht.service import oauth_service, webhooks_notify
from hgsrht.types import User, OAuthToken
import hgsrht.webhooks # makes valid the global
from jinja2 import FileSystemLoader, ChoiceLoader

db = DbSession(cfg("hg.sr.ht", "connection-string"))
db.init()

class HgApp(SrhtFlask):
    def __init__(self):
        super().__init__("hg.sr.ht", __name__,
                oauth_service=oauth_service)

        from hgsrht.blueprints.auth import auth
        from hgsrht.blueprints.public import public
        from hgsrht.blueprints.api import register_api
        from hgsrht.blueprints.api.data import data
        from hgsrht.blueprints.repo import repo
        from hgsrht.blueprints.manage import manage
        from hgsrht.blueprints.extramanage import extramanage
        from hgsrht.blueprints.internal import internal
        from srht.graphql import gql_blueprint

        self.register_blueprint(auth)
        self.register_blueprint(public)

        register_api(self)
        self.register_blueprint(data)
        self.register_blueprint(repo)
        self.register_blueprint(manage)
        self.register_blueprint(extramanage)
        self.register_blueprint(webhooks_notify)
        self.register_blueprint(internal)
        self.register_blueprint(gql_blueprint)

        from hgsrht.webhooks import webhook_metrics_collector
        self.metrics_registry.register(webhook_metrics_collector)

        @self.context_processor
        def inject():
            notice = session.get("notice")
            if notice:
                del session["notice"]
            return {
                "stat": stat,
                "humanize": humanize,
                "notice": notice,
                "path_join": os.path.join,
                "scmname": "hg",
                "scmreflabel": "rev",
                "lookup_user": self.lookup_user
            }

        choices = [self.jinja_loader, FileSystemLoader(os.path.join(
            os.path.dirname(__file__), "templates"))]
        self.jinja_loader = ChoiceLoader(choices)

        self.url_map.strict_slashes = False

    def lookup_user(self, email):
        return User.query.filter(User.email == email).one_or_none()

app = HgApp()
# Add a middleware that will capture requests that look like mercurial
# requests (like https cloning) in order to pass them to hgweb.
app.wsgi_app = HgWebShim(app.wsgi_app)
