# hgsrht extension
#
# Mercurial extension used internally by hgsrht, sourcehut.org's Mercurial
# hosting service.

from mercurial.i18n import _
from mercurial import scmutil, registrar, pycompat

cmdtable = {}
command = registrar.command(cmdtable)

"""facilities for hgsrht"""

@command(b"manifest2",
         [(b'r', b'rev', b'', _(b'revision to display'), _(b'REV')),
          (b'I', b'include', b'', _(b'path to include'), _(b'PATH'))],
         _(b'[-r REV]'))
def manifest2(ui, repo, rev=None, **opts):
    """like the manifest command, but with more info"""
    char = {b'l': b'@', b'x': b'*', b'': b'', b't': b'd'}
    mode = {b'l': b'644', b'x': b'755', b'': b'644', b't': b'755'}

    opts = pycompat.byteskwargs(opts)
    ctx = scmutil.revsingle(repo, rev)
    mf = ctx.manifest()

    incpath = opts.get(b'include')
    len_incpath = len(incpath)

    cut_pos = 0
    slash_pos = len_incpath
    if len_incpath > 0:
        if incpath.endswith(b'/'):
            slash_pos -= 1
        cut_pos = slash_pos + 1

    items = {}

    for f, fctx in mf.items():
        # Eliminate anything that's not remotely close to what we want.
        if len_incpath > 0 and f[:len_incpath] != incpath:
            continue

        # Now figure out if we have the exact thing we want (if we were
        # given a file name), something that's inside what we want (if
        # we were given a directory name), or something that just starts
        # the same but isn't what we want.
        if len_incpath > 0:
            if f == incpath:
                # The file we want!
                items[b'.'] = f
                break

            if f[slash_pos:cut_pos] != b'/':
                # Something whose name starts the same, but isn't in the
                # directory we wanted.
                continue

        remain = f[cut_pos:]
        elements = remain.split(b'/')
        if len(elements) == 1:
            # This is a file that's at the nesting level we want.
            items[remain] = f
        else:
            # This is a file that's in a sub-directory, so we just
            # record that directory's existence if needed.
            first_elem = elements[0]
            if first_elem not in items:
                items[first_elem] = None  # Denotes a non-empty directory.

    for rel, path in items.items():
        if path is not None:
            fctx = ctx.filectx(path)
            fl = fctx.flags()
            fsize = fctx.size()
            ui.write(b'%s %1s %d %s\n' % (mode[fl], char[fl], fsize, rel))
        else:
            ui.write(b'755 d 0 %s\n' % rel)
